%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read_vmr_v3and4_bv20.m
% Purpose: read BrainVoyager *.vmr file (v3 and v4)(VMR = Volume based anatomical MRI data)
% in 8 bit or 16 bit format; fields will be in Workspace.
% Activate: load this script in Matlab and press 'run'
% Note: script does not take endianness/byte ordering into account
% Latest update: Nov 2005, mod 02-11-17
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%clear; % clear workspace
h = msgbox('This script will read a BrainVoyager *.vmr file', 'Read BrainVoyager VMR file');
uiwait(h);
[FileName,PathName] = uigetfile('*.vmr','Please select the BrainVoyager *.vmr file...');
vmrFile = fullfile(PathName, FileName);
fid=fopen(vmrFile,'rb');
% VMR header
VMRversion = fread(fid,[1 1], 'ushort');
DimX = fread(fid,[1 1],'ushort');
DimY = fread(fid,[1 1],'ushort');
DimZ = fread(fid,[1 1],'ushort');
% VMR data
vmrVolume1D = fread(fid,DimZ*DimY*DimX, 'uint8'); % normal *.vmr
vmrVolume3D= zeros(DimX, DimY, DimZ);
vmrVolume3D(:) = vmrVolume1D(:);
clear vmrVolume1D;

% scanner information, also saved in *.pos file format 3
Xoffset = fread(fid,[1 1],'ushort');
Yoffset = fread(fid,[1 1],'ushort');
Zoffset = fread(fid,[1 1],'ushort');
framing = fread(fid,[1 1],'ushort');
PosInfoVerified = fread(fid,[1 1], 'int32');
CoordinateSystem = fread(fid,[1 1], 'int32');
Slice1CenterX = fread(fid,[1 1], 'float32');
Slice1CenterY = fread(fid,[1 1], 'float32');
Slice1CenterZ = fread(fid,[1 1], 'float32');
SliceNCenterX = fread(fid,[1 1], 'float32');
SliceNCenterY = fread(fid,[1 1], 'float32');
SliceNCenterZ = fread(fid,[1 1], 'float32');
RowDirX = fread(fid,[1 1], 'float32');
RowDirY = fread(fid,[1 1], 'float32');
RowDirZ = fread(fid,[1 1], 'float32');
ColDirX = fread(fid,[1 1], 'float32');
ColDirY = fread(fid,[1 1], 'float32');
ColDirZ = fread(fid,[1 1], 'float32');
NrOfRows = fread(fid,[1 1], 'int32');
NrOfCols = fread(fid,[1 1], 'int32');
FoVRows = fread(fid,[1 1], 'float32');
FoVCols = fread(fid,[1 1], 'float32');
SliceThickness = fread(fid,[1 1], 'float32');
GapThickness = fread(fid,[1 1], 'float32');

% performed transformations on file
NrOfTransformations = fread(fid,[1 1], 'int32');
TransformValsContainer = cell(NrOfTransformations);
nameTransformationContainer = cell(NrOfTransformations);
nameSourceFileContainer = cell(NrOfTransformations);
for i=1:NrOfTransformations
    rawChar=1; % get name of transformation (ends with 0)
    position=1;
    while rawChar~=0
        rawChar=fread(fid,[1 1],'uchar');
        realChar=char(rawChar);
        nameTransformation(position)=realChar;
        position=position+1;
    end
    nameTransformationContainer{i} = cellstr(nameTransformation);
    % Transformation types:
    % 1: rigid body with 9 params (3 translation, 3 rotation, 3 scaling
    % params)(?)
    % 2: affine transformation with 16 params (4x4 matrix)
    % 3: --
    % 4: talairach transformation (24 params)
    % 5: talairach to BrainVoyager coordinate system transformation
    TypeOfTransformation(i) = fread(fid,[1 1], 'int32');
    rawChar=1; % % get name of source file (ends with 0)
    position=1;
    while rawChar~=0
        rawChar=fread(fid,[1 1],'uchar');
        realChar=char(rawChar);
        nameSourceFile(position)=realChar;
        position=position+1;
    end
    nameSourceFileContainer{i} = cellstr(nameSourceFile);
    NrOfTransformationVals = fread(fid,[1 1], 'int32');
    TransformVals = fread(fid,[1 NrOfTransformationVals], 'float');
    TransformValsContainer{i} = TransformVals;
end
RadiologicalConvention = fread(fid,[1 1], 'uint8');
if (VMRversion == 4)
    ReferenceSpaceFlag = fread(fid,[1 1], 'uint8');  %   (new in v4)
end
eofstat = feof(fid);
disp(['eofstat: ' eofstat])
if (eofstat~=1)
    VoxResX = fread(fid,[1 1], 'float');
    VoxResY = fread(fid,[1 1], 'float');
    VoxResZ = fread(fid,[1 1], 'float');
    VoxelResInTal = fread(fid,[1 1], 'uint8');
    VoxelResVerified = fread(fid,[1 1], 'uint8');
else
    VoxResX = -1.0;
    VoxResY = -1.0;
    VoxResZ = -1.0;
    VoxelResInTal = 0;
    VoxelResVerified = 0;
end
origMinIntensity = fread(fid,[1 1], 'int32');
origMeanIntensity = fread(fid,[1 1], 'int32');
origMaxIntensity = fread(fid,[1 1], 'int32');
eofstat = feof(fid) % check if end of file reached
fclose(fid);

